#!/usr/bin/python3

import json
import os
import requests

from . import hetzner_dns_helpers as helpers
from .record_get import record_get


def delete_record_by_id(hetzner_dns_token, record_id):
    try:
        response = requests.delete(
            url=f'https://dns.hetzner.com/api/v1/records/{record_id}',
            headers={'Auth-API-Token': hetzner_dns_token})

        decoded_response = response.content.decode('utf-8')
        response_dict = json.loads(decoded_response)

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        # when running via the terminal, print output to console
        if __name__ == '__main__':
            print("OK")

        return "OK"

    except requests.exceptions.RequestException as err:
        helpers.handle_request_exception(err)


def record_delete(hetzner_dns_token=None,
                  record_id=None,
                  record_ids=None,
                  zone_id=None,
                  zone_name=None,
                  name=None,
                  ttl=None,
                  record_type=None,
                  value=None,
                  first_record_only=None,
                  delete_multiple_records=False,
                  search_all_zones=False):
    """
    Delete an existing record.
    https://dns.hetzner.com/api-docs/#operation/DeleteRecord

    Required* Parameters: One of: record_id or zone_id or zone_name

    Optional Parameters:
      Filters: record_type, name, value, ttl,
      Options: delete_multiple_records, first_record_only, search_all_zones*


    * This function will raise an exception if multiple records are
      returned, *unless* the `first_record_only` *or*
      `delete_multiple_records` parameters are truthy.

    - Deletions can be performed directly with 'record_id', or indirectly
      via a lookup that uses any combination of 'name', 'ttl',
      'record_type', 'value', 'zone_id', and/or 'zone_name'.

    - If doing a lookup, you must either specify a 'zone_id' or
      'zone_name', or assign a truthy value to 'search_all_zones'.

    * hetzner_dns_token *MUST* be passed in args or as environment
      variable (HETZNER_DNS_TOKEN). You can get a DNS API token
      here: https://dns.hetzner.com/settings/api-token

    - If using Bash environment variables, ensure that values are assigned
      in ALL_CAPS.
          - e.g. zone_id in Python -> ZONE_ID in environment variable
    """
    if hetzner_dns_token is None:
        # get token from environment variable
        hetzner_dns_token = os.environ['HETZNER_DNS_TOKEN']

    if record_id is None and os.environ.get('RECORD_ID'):
        # get record_id from environment variable
        record_id = os.environ['RECORD_ID']

    if record_ids is None and os.environ.get('RECORD_IDS'):
        # get record_ids from environment variable
        record_ids = os.environ['RECORD_IDS']

    if not first_record_only\
            and os.environ.get('FIRST_RECORD_ONLY'):
        # get first_record_only from environment variable
        first_record_only = os.environ['FIRST_RECORD_ONLY']

    if not search_all_zones\
            and os.environ.get('SEARCH_ALL_ZONES'):
        # get search_all_zones from environment variable
        search_all_zones = os.environ['SEARCH_ALL_ZONES']

    if not delete_multiple_records\
            and os.environ.get('DELETE_MULTIPLE_RECORDS'):
        # get delete_multiple_records from environment variable
        delete_multiple_records = os.environ['DELETE_MULTIPLE_RECORDS']

    # do an indirect lookup of all relevant records that match the parameters
    # and allow a single match to be returned
    if not record_id:
        if zone_id is None and os.environ.get('ZONE_ID'):
            # get zone_id from environment variable
            zone_id = os.environ['ZONE_ID']

        if zone_name is None and os.environ.get('ZONE_NAME'):
            # get zone_name from environment variable
            zone_name = os.environ['ZONE_NAME']

        if name is None and os.environ.get('NAME'):
            # get name from environment variable
            name = os.environ.get('NAME', None)

        if ttl is None and os.environ.get('TTL'):
            # get ttl from environment variable
            ttl = int(os.environ['TTL'])

        if record_type is None and os.environ.get('TYPE'):
            # get record_type from environment variable
            record_type = os.environ['RECORD_TYPE']\
                if os.environ.get('RECORD_TYPE') else os.environ['TYPE']

        if value is None and os.environ.get('VALUE'):
            # get value from environment variable
            value = os.environ['VALUE']

        # this method will return a string if one record is returned,
        # and a list if multiple records are returned
        record_get_id_result =\
            record_get(zone_id=zone_id,
                       zone_name=zone_name,
                       name=name,
                       ttl=ttl,
                       record_type=record_type,
                       value=value,
                       id_only=True,
                       first_record_only=first_record_only,
                       allow_multiple_records=delete_multiple_records,
                       search_all_zones=search_all_zones)

        if type(record_get_id_result) == list:
            records_count = len(record_get_id_result)

            # do not delete multiple records unless explicitly requested
            if not delete_multiple_records:
                error_message = f"Found {records_count} records. To "\
                    "delete multiple records via a query, assign a truthy "\
                    "value to 'delete_multiple_records', or use "\
                    "'first_record_only' to get only the "\
                    "first record. Capitalize these values if using "\
                    "environment variables."
                helpers.exit_with_error(error_message)

            else:
                record_ids = record_get_id_result
        else:
            if record_get_id_result:
                record_id = record_get_id_result
            else:
                helpers.exit_with_error("record not found")

    if record_id and record_ids:
        helpers.exit_with_error(
            "Cannot use 'record_id' and 'record_ids' at the same time.")
    elif record_id:
        delete_record_by_id(hetzner_dns_token, record_id)
    elif record_ids:
        for record_id_value in record_ids:
            delete_record_by_id(hetzner_dns_token, record_id_value)
    else:
        helpers.exit_with_error("No 'record_id' or 'record_ids' found.")


if __name__ == '__main__':
    record_delete()



#!/usr/bin/python3

import json
import os
import sys
import requests

from . import hetzner_dns_helpers as helpers
from .zone_list import zone_list


def zone_delete(hetzner_dns_token=None,
                zone_id=None,
                zone_name=None,
                name=None):
    """
    Delete an existing zone.

    Required Parameters: One of: `zone_id` or `zone_name/name`


    - Lookups can be performed using 'zone_name/name' *OR* 'zone_id'.

    * hetzner_dns_token *MUST* be passed in args or as environment
      variable (HETZNER_DNS_TOKEN). You can get a DNS API token
      here: https://dns.hetzner.com/settings/api-token

      - TIP: zone_create and zone_delete allow ZONE_NAME and NAME to
        be used interchangeably.

    - If using Bash environment variables, ensure that values are assigned
      in ALL_CAPS.
          - e.g. zone_id in Python -> ZONE_ID in environment variable
    """
    if os.environ.get('SHOW_HELP'):
        # print the docstring and exit
        print(zone_delete.__doc__)
        sys.exit(0)

    if hetzner_dns_token is None:
        # get token from environment variable
        hetzner_dns_token = os.environ['HETZNER_DNS_TOKEN']

    if zone_name is None:
        # get name from environment variable
        zone_name = os.environ['NAME']\
            if os.environ.get('NAME') else os.environ.get('ZONE_NAME', None)
    elif name:
        # allow zone_name and name to be used interchangeably
        # for zone_create and zone_delete
        zone_name = name

    if os.environ.get('ZONE_ID'):
        zone_id = os.environ['ZONE_ID']

    # if zone_name exists, use it to obtain zone (skip if zone_id exists)
    if zone_name and not zone_id:

        # get list of zones
        response_dict = zone_list()

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        # check for matching zone
        dns_zones = response_dict['zones']
        for zone in dns_zones:
            if zone['name'] == zone_name:
                zone_id = zone['id']
                break

        # if no matching zone found, halt and notify of error
        if zone_id is None:
            helpers.exit_with_error("zone not found")

    # raise an exception if no zone_id or zone_name have been passed
    if not zone_id and not zone_name:
        helpers.exit_with_error("Must specify one of: zone_id, zone_name")

    if zone_id is None:
        # get zone_id from environment variable
        zone_id = os.environ['ZONE_ID']

    try:
        response = requests.delete(
            url=f'https://dns.hetzner.com/api/v1/zones/{zone_id}',
            headers={'Auth-API-Token': hetzner_dns_token})

        decoded_response = response.content.decode('utf-8')
        response_dict = json.loads(decoded_response)

        # check response for errors
        helpers.check_response_for_errors(response_dict)

        # when running via the terminal, print output to console
        if __name__ == '__main__':
            print("OK")
            sys.exit(0)  # exit successfully

        return "OK"

    except requests.exceptions.RequestException as err:
        helpers.handle_request_exception(err)


if __name__ == '__main__':
    zone_delete()



#!/usr/bin/python3

import json
import os
import requests
import sys

from . import hetzner_dns_helpers as helpers
from .record_list import record_list
from .zone_get import zone_get


def record_get(hetzner_dns_token=None,
               record_id=None,
               zone_id=None,
               zone_name=None,
               record_type=None,
               name=None,
               value=None,
               ttl=None,
               first_record_only=False,
               allow_multiple_records=False,
               search_all_zones=False,
               id_only=False):
    """
    Get info about an existing record.
    https://dns.hetzner.com/api-docs/#operation/GetRecord

    Required* Parameters: One of: `record_id` or `zone_id` or `zone_name`

    Optional Parameters:
      Filters: record_type, name, value, ttl,
      Formats: id_only
      Options: first_record_only, allow_multiple_records, search_all_zones


    * This function will raise an exception if multiple records are
      returned, *unless* the `first_record_only` *or*
      `allow_multiple_records` parameters are truthy.

    - Lookups can be performed directly with 'record_id', or indirectly
      using a combination of 'name', 'ttl', 'record_type', 'value',
      'zone_id', and 'zone_name'.

    - If doing an indirect lookup, you must either specify a 'zone_id' or
      'zone_name', or assign a truthy value to 'search_all_zones'.

    - If indirect lookups are performed, an exception will be raised if
      multiple records are returned, *UNLESS* you specify a truthy
      value for the 'allow_multiple_records' or the 'first_record_only'
      parameters.

    - If 'id_only' passed in args or as environment variable (ID_ONLY),
      return just the record ID if one exists.

      - If 'id_only' and 'allow_multiple_records' are truthy, then return
        a list of record IDs.

    * hetzner_dns_token *MUST* be passed in args or as environment
      variable (HETZNER_DNS_TOKEN). You can get a DNS API token
      here: https://dns.hetzner.com/settings/api-token

    - If using Bash environment variables, ensure that values are assigned
      in ALL_CAPS.
          - e.g. zone_id in Python -> ZONE_ID in environment variable
    """
    if hetzner_dns_token is None:
        # get token from environment variable
        hetzner_dns_token = os.environ['HETZNER_DNS_TOKEN']

    if record_id is None and os.environ.get('RECORD_ID'):
        # get record_id from environment variable
        record_id = os.environ['RECORD_ID']

    if not id_only and os.environ.get('ID_ONLY'):
        # get id_only from environment variable
        id_only = os.environ['ID_ONLY']

    # if record_id exists, do a direct lookup to obtain the record
    if record_id:
        # get response
        try:
            response = requests.get(
                url=f'https://dns.hetzner.com/api/v1/records/{record_id}',
                headers={'Auth-API-Token': hetzner_dns_token,
                         'Content-Type': 'application/json; charset=utf-8'})

            decoded_response = response.content.decode('utf-8')
            response_dict = json.loads(decoded_response)

            # check response for errors
            helpers.check_response_for_errors(response_dict)

            # if id_only, return just the record ID
            if id_only:
                if __name__ == '__main__':
                    print(response_dict['record']['id'])
                    sys.exit(0)  # exit successfully
                else:
                    return response_dict['record']['id']

            # when running via the terminal, print output to console then exit
            if __name__ == '__main__':
                print(decoded_response)
                sys.exit(0)  # exit successfully

            return response_dict

        except requests.exceptions.RequestException as err:
            helpers.handle_request_exception(err)

    # do an indirect lookup of all relevant records and filter via given params
    if zone_id is None and os.environ.get('ZONE_ID'):
        # get zone_id from environment variable
        zone_id = os.environ['ZONE_ID']

    if zone_name is None and os.environ.get('ZONE_NAME'):
        # get zone_name from environment variable
        zone_name = os.environ['ZONE_NAME']

    if record_type is None and os.environ.get('TYPE'):
        # get record_type from environment variable
        record_type = os.environ['RECORD_TYPE']\
            if os.environ.get('RECORD_TYPE') else os.environ['TYPE']

    if name is None and os.environ.get('NAME'):
        # get name from environment variable
        name = os.environ.get('NAME', None)

    if value is None and os.environ.get('VALUE'):
        # get value from environment variable
        value = os.environ['VALUE']

    if ttl is None and os.environ.get('TTL'):
        # get ttl from environment variable
        ttl = int(os.environ['TTL'])

    if not first_record_only\
            and os.environ.get('FIRST_RECORD_ONLY'):
        # get first_record_only from environment variable
        first_record_only = os.environ['FIRST_RECORD_ONLY']

    if not allow_multiple_records\
            and os.environ.get('ALLOW_MULTIPLE_RECORDS'):
        # get allow_multiple_records from environment variable
        allow_multiple_records = os.environ['ALLOW_MULTIPLE_RECORDS']

    if first_record_only and allow_multiple_records:
        helpers\
            .exit_with_error("This combination of options doesn't make sense.")

    if not search_all_zones\
            and os.environ.get('SEARCH_ALL_ZONES'):
        # get search_all_zones from environment variable
        search_all_zones = os.environ['SEARCH_ALL_ZONES']

    # BEGIN validation #

    # if no record_id exists, ensure that zone_name or zone_id exist,
    # in order to prevent pulling records from multiple zones
    if not record_id and not zone_name and not zone_id\
            and not search_all_zones:
        error_message = "In order to prevent records from being pulled from "\
            "more than one zone, you must specify one (or more) of: "\
            "record_id, zone_id, or zone_name. You can override this "\
            "behavior by assigning a truthy value to 'search_all_zones'."
        helpers.exit_with_error(error_message)

    # ensure that one or more optional parameters exist before doing
    # an indirect lookup
    if not record_id and not search_all_zones and not zone_name\
            and not name and not ttl and not record_type and not value:
        error_message =\
            "You must provide a record_id or one or more of the following: "\
            "name, ttl, record_type (environment variable: TYPE), or value, "\
            "*OR* you must set a truthy value for 'search_all_zones.'"
        helpers.exit_with_error(error_message)

    # if zone_name passed, lookup the zone that matches it to get zone_id
    if zone_name:
        zone_name_id = zone_get(zone_name=zone_name, id_only=True)
        if not zone_id:
            zone_id = zone_name_id
        elif zone_id != zone_name_id:
            error_message =\
                "The zone_id you entered does not match the zone_id of "\
                "the zone_name you entered."
            if __name__ == '__main__':
                print(f"Error: {error_message}")
                sys.exit(1)  # exit with error
            else:
                raise ValueError(error_message)

    # END validation #

    records = record_list(hetzner_dns_token=hetzner_dns_token,
                          zone_id=zone_id)['records']

    # iterate over the given parameters, adding any matching records that
    # are not yet in the list
    if (zone_id or zone_name) and\
            (not name and not ttl and not record_type and not value):
        filtered_records = records
    else:
        filtered_records = []
        for record in records:
            # if record does not meet any one qualifying criterion,
            # then skip over it and continue the loop
            if name and record['name'] != name:
                continue
            if ttl and record['ttl'] != ttl:
                continue
            if record_type and record['type'] != record_type:
                continue
            if value and record['value'] != value:
                continue
            # if the record has not fail any of the qualifications,
            # then add it to the list
            filtered_records.append(record)

    # if no records found, return empty dictionary
    if len(filtered_records) == 0:
        # when running via the terminal, print output to console then exit
        if __name__ == '__main__':
            print({})
            sys.exit(0)  # exit successfully
        return {}

    # if one record found, return it
    if len(filtered_records) == 1:
        # if id_only, return just the record ID
        if id_only:
            if __name__ == '__main__':
                print(filtered_records[0]['id'])
                sys.exit(0)  # exit successfully
            else:
                return filtered_records[0]['id']

        # when running via the terminal, print output to console then exit
        if __name__ == '__main__':
            print(json.dumps(filtered_records[0]))
            sys.exit(0)  # exit successfully
        return filtered_records[0]

    # handle multiple records
    if len(filtered_records) > 1:
        records_count = len(filtered_records)
        plural = "s" if records_count != 1 else ""
        if first_record_only:
            # if id_only, return just the record ID
            if id_only:
                if __name__ == '__main__':
                    print(filtered_records[0]['id'])
                    sys.exit(0)  # exit successfully
                else:
                    return filtered_records[0]['id']

            # when running via the terminal, print output to console then exit
            if __name__ == '__main__':
                print(json.dumps(filtered_records[0]))
                sys.exit(0)  # exit successfully
            return filtered_records[0]
        elif id_only and allow_multiple_records:
            id_list = []
            for record in filtered_records:
                id_list.append(record['id'])

            if __name__ == '__main__':
                print(json.dumps(id_list))
                sys.exit(0)  # exit successfully
            else:
                return id_list

        elif allow_multiple_records and not id_only:
            # when running via the terminal, print output to console,
            # then exit
            if __name__ == '__main__':
                print(json.dumps(filtered_records))
                sys.exit(0)  # exit successfully
            return filtered_records
        else:
            error_message =\
                f"Found {records_count} record{plural}. Assign a truthy "\
                "value to 'allow_multiple_records' To get all relevant "\
                "records, or use 'first_record_only' to get only the "\
                "first record. Capitalize these values if using "\
                "environment variables."
            helpers.exit_with_error(error_message)


if __name__ == '__main__':
    record_get()
